import { defineComponent , reactive} from "vue";
import { string } from "vue-types";
import { useCompData } from ".";
import { useEditor } from "../../../..";
import { useCompRef } from "../hooks";
import { css } from "@linaria/core";
import { PageMusic } from "./PageMusic";
import { MusicOptions } from "./localMusic";

export const Component = defineComponent({
  props: {
    compId: string().isRequired,
  },
  setup(props, { slots }) {
    const editor = useEditor();
    const { helper, store, controls } = editor;
    const compRef = useCompRef(props.compId);

  

    return () => {
      const comp = useCompData(props.compId);
      const { children, layout, value } = comp;
      const compMusic = value.music || "";
      const curValue = MusicOptions.find((e) => {
        return e.value == compMusic;
      });
      const style = helper.createStyle(layout || { size: [750] }, comp);
      if (comp.value.pageMode == "short") {
        style.height = "100%";
      }

      let contextStyle:any = null;
      if (store.rootPage.value.pageMode == "short" && !store.isEditMode) {
        contextStyle = { transform: `translate(0, ${ -store.shortPage.index * window.innerHeight + store.shortPage.offset}px`};
        if (!store.shortPage.isMoving) {
          contextStyle.transition = "transform .4s ease-out";
        }
      }
      console.log("curr card=>", store.currStreamCardId, store.currStreamCard?.children.default?.length);

      return (
        <div
          ref={compRef}
          style={style}
          class={[comp.value.pageMode != "short" && "!h-auto", editor.store.isEditMode ? pageEditStyle : ""]}
        >
          {
             store.isDisplay && <div class="relative z-1000" style={contextStyle}>
             {slots.Container?.(
               children.default.map((compId) => {
                 const comp = helper.findComp(compId);
                 if (!comp) return;
                 return slots.CompItem?.(comp);
               })
             )}
             {curValue?.value && !editor.store.isEditMode && <PageMusic />}
           </div>
 
          }
          {
            store.isEditMode && store.currStreamCard && <div class="relative z-1000" style={contextStyle}>
            {
              slots.Container?.( [slots.CompItem?.(store.currStreamCard)])
            }
          </div>

          }

        </div>
      );
    };
  },
});

const pageEditStyle = css`
  box-shadow: 0 0 0 3000px rgba(0, 0, 0, 0.4);
`;
