import { IconAdd, IconMove } from "@/assets/icons";
import { CompObject } from "@/modules/editor/controllers/SelectCtrl/compObj";
import { Design_Page_Size } from "@/modules/editor/dicts/CompOptions";
import { DesignComp } from "@/modules/editor/objects/DesignTemp/DesignComp";
import { css, cx } from "@linaria/core";
import { IconDelete } from "@queenjs/icons";
import "animate.css";
import { defineComponent, onMounted, onUnmounted, reactive, ref } from "vue";
import { bool, string } from "vue-types";
import { useEditor } from "../../..";
import { useCompEditLayerRef, useCompRef } from "./hooks";

let intersectionObserver: any;
export const View = defineComponent({
  props: {
    compId: string().isRequired,
    editlayer: bool().def(true),
    showMask: bool().def(false),
  },
  emits: ["dblclick", "click"],
  setup(props, { slots, emit }) {
    const { store, actions, helper, controls } = useEditor();
    const compRef = useCompRef(props.compId);
    const editorLayerRef = props.editlayer
      ? useCompEditLayerRef(props.compId)
      : ref();

    const isLongPage = controls.screenCtrl.isLongPage;

    const state = reactive({
      showAnimation: false,
    });

    onUnmounted(() => {
      intersectionObserver.disconnect();
    });

    onMounted(() => {
      intersectionObserver = new IntersectionObserver((entries) => {
        if (entries[0].intersectionRatio <= 0) {
          if (isLongPage) return;
          state.showAnimation = false;
        } else {
          state.showAnimation = true;
        }
      });
      intersectionObserver.observe(compRef.value);
    });

    return () => {
      const comp = helper.findComp(props.compId);
      if (!comp) return store.isEditMode ? <div>无效组件</div> : null;
      const isStreamCard = helper.isStreamCard(props.compId);
      const isGroupComp = helper.isGroupComp(props.compId);
      const isStreamChild = helper.isStreamCardChild(props.compId);
      const cardId = helper.getCompTrees(props.compId)[1]?.id;
      const cardIndex = store.streamCardIds.indexOf(cardId)
      const obj = new CompObject(store.compMap[props.compId]);
      const m = obj.worldTransform.clone();
      m.invert();

      let isFocus =
        store.isEditMode &&
        store.selected.length > 1 &&
        store.lastSelected == props.compId;
      isFocus = isFocus || store.currCompId == props.compId;

      const style = helper.createStyle(comp.layout, comp);
      const page = helper.findRootComp() as DesignComp;

      if (isStreamCard && page.value.pageMode == "short" && store.isDisplay) {
        //  const cards = store.streamCardIds;
        //  const index = cards.indexOf(props.compId)
        //  style.position = "absolute";
        //  style.top = index * 100  + "vh";
        style.height = helper.designToNaturalSize(Design_Page_Size[1], {
          adaptiveH: true,
        });
      }

      // if (comp.anim && comp.anim.transition) {
      //   style.transition = "all 1s ease-out";
      // }

      let aniStyles = "";
      if (
        comp.layout.anim &&
        state.showAnimation&&
        ((!props.showMask && store.isEditMode) || !store.isEditMode)
      ) {
        aniStyles = `animate__animated animate__${comp.layout.anim} animate__delay-0.1s`;
      }

      function RenderPre() {
        if (
          store.currStreamCardId != props.compId ||
          !isStreamCard ||
          !store.isEditMode
        )
          return;
        let i = store.streamCardIds.indexOf(props.compId);
        if (i > 0) {
          const c = helper.findComp(store.streamCardIds[i - 1]) as DesignComp;
          const PreComp = controls.compUICtrl.state.components.get(
            c.compKey
          ) as any;
          return (
            <div
              class="absolute w-full h-150px"
              style={{
                bottom: style.height,
              }}
            >
              <div class="absolute w-full h-150px pointer-events-none overflow-hidden">
                <PreComp.Component
                  compId={c.id}
                  key={c.id}
                  style={{ position: "absolute", bottom: "0" }}
                />
              </div>
              <div class={maskStyleUp}></div>
              <div class={[divideStyle, "bottom"]}>
                <span class="tip">上一页分割线</span>
              </div>
            </div>
          );
        }
      }

      function RenderAfter() {
        if (
          store.currStreamCardId != props.compId ||
          !isStreamCard ||
          !store.isEditMode
        )
          return;

        let i = store.streamCardIds.indexOf(props.compId);
        if (i < store.streamCardIds.length - 1) {
          const c = helper.findComp(store.streamCardIds[i + 1]) as DesignComp;
          const AfterComp = controls.compUICtrl.state.components.get(
            c.compKey
          ) as any;
          return (
            <div class="relative">
              <div class="h-150px pointer-events-none overflow-hidden">
                <AfterComp.Component compId={c.id} key={c.id} />
              </div>

              <div class={maskStyle}></div>
              <div class={[divideStyle, "top"]}>
                <span class="tip">下一页分割线</span>
              </div>
            </div>
          );
        }
      }

      return (
        <div
          ref={compRef}
          class={[
            viewStyle,
            store.isEditMode &&
              controls.selectCtrl._state != 0 &&
              editCompStyle,
            isGroupComp && groupCompCls,
            store.currStreamCardId == props.compId && CurrCompStyle,
            isFocus && AnchorCompStyle,
          ]}
          style={style}
          onClick={(e) => {
            if (!store.isEditMode) {
              e.stopPropagation();
              emit("click");
            }
          }}
          onDblclick={() => emit("dblclick")}
        >
          <div
            class={cx(
              comp.layout.anim ? (aniStyles ? aniStyles : "opacity-0") : ""
            )}
            onMousemove={(e) => {
              if (
                !store.isEditMode ||
                !controls.dragAddCtrl.dragingCompKey ||
                !helper.isStreamCard(props.compId)
              )
                return;
              actions.pickComp(props.compId);
            }}
          >
            {slots.default?.()}
          </div>

          {/* {store.isEditMode &&
            isStreamCard &&
            store.currStreamCardId == props.compId && (
              <Hudop compId={props.compId} />
            )} */}

          {store.isEditMode && props.editlayer && (
            <div ref={editorLayerRef} class={editAreaStyle}></div>
          )}

          {props.showMask && isLongPage && (
            <>
              {RenderPre()}
              {RenderAfter()}
            </>
          )}
          {props.showMask && !isLongPage && (
            <div
              class={cx(
                safeAreaStyles,
                "absolute left-0 top-1/2 w-1/1 translate -translate-y-1/2 transform pointer-events-none"
              )}
              style={{
                height: `${controls.screenCtrl.state.safeFactor * 100}%`,
              }}
            >
              <div class={[divideStyle, "top"]}>
                <span class="tip"> 安全线</span>
              </div>
              <div class={[divideStyle, "bottom"]}>
                <span class="tip"> 安全线</span>
              </div>
            </div>
          )}
        </div>
      );
    };
  },
});

export const Hudop = defineComponent({
  props: {
    compId: string().isRequired,
  },
  setup(props) {
    const { store, actions, helper, controls } = useEditor();

    const opref = ref();

    onMounted(() => {
      opref.value.editable = "hudop";
    });

    return () => (
      <div class="hudop shadow" ref={opref}>
        {store.streamCardIds.length > 1 && (
          <IconMove
            class="draganchor"
            onMousedown={() => actions.pickComp(props.compId)}
          />
        )}
        {store.streamCardIds.length > 1 && (
          <IconDelete
            onClick={(e: any) => {
              e.stopPropagation();
              actions.removeStreamCard(props.compId);
            }}
          />
        )}
        <IconAdd
          onClick={(e: any) => {
            e.stopPropagation();
            const index = store.streamCardIds.indexOf(props.compId) + 1;
            actions.addCompToDesign("Container", index);
          }}
        />
      </div>
    );
  },
});

const viewStyle = css`
  position: relative;
  font-size: 0;
  cursor: pointer;

  > :first-child {
    width: 100%;
    height: 100%;
  }

  .hudop {
    position: absolute;
    top: 0px;
    left: -46px;
    background-color: white;
    flex-direction: column;
    color: black;
    display: flex;
    font-size: 12px;
    width: 28px;
    align-items: center;
    border-radius: 4px;
    z-index: 997;
    .inficon {
      padding: 8px;
    }
  }
`;

const editCompStyle = css`
  &:hover {
    outline: 2px dashed @inf-primary-color;
  }
`;

const AnchorCompStyle = css`
  outline: 2px dashed @inf-primary-color;
`;

const CurrCompStyle = css`
  position: relative;
  outline: 1px solid @inf-primary-color;
  z-index: 998;
`;

//box-shadow: 0 0 0 3000px rgba(0, 0, 0, 0.5);

const groupCompCls = css`
  outline: 2px solid @inf-primary-color !important;
`;

const editAreaStyle = css`
  position: absolute;
  top: 0;
  left: 0;
  width: 100%;
  height: 100%;
  pointer-events: none;
`;

// const editAreaTestStyle = css`
//   position: absolute;
//   top: 0;
//   left: 0;
//   width: 100px;
//   height: 100px;
//   background-color: red;
// `;
const maskStyle = css`
  background: rgba(0, 0, 0, 0.3);
  position: absolute;
  left: 0;
  top: 0;
  width: 100%;
  height: 100%;
`;

const maskStyleUp = css`
  background: rgba(0, 0, 0, 0.3);
  position: absolute;
  left: 0;
  top: 0;
  width: 100%;
  height: 100%;
`;

const divideStyle = css`
  position: absolute;
  left: -10%;
  width: 120%;
  /* border: 1px dashed #EB684E;
  border-dasharray: 5px 10px; */
  height: 1px;
  background-image: linear-gradient(
    to right,
    #eb684e 0%,
    #eb684e 50%,
    transparent 50%
  );
  background-size: 8px 1px;
  background-repeat: repeat-x;

  &.top {
    top: 0;
  }

  &.bottom {
    bottom: 0;
  }
  .tip {
    margin-left: 10px;
    font-size: 12px;
    position: absolute;
    left: 100%;
    width: 80px;
    top: -10px;
    color: #eb684e;
  }
`;

const safeAreaStyles = css`
  box-shadow: 0 0 0 3000px rgba(0, 0, 0, 0.3);
`;
