import { IconPicker } from "@/assets/icons";
import iro from "@jaames/iro";
import { css, cx } from "@linaria/core";
import { Input, InputNumber } from "ant-design-vue";
import Color from "color";
import { defineComponent, nextTick, onMounted, reactive, ref } from "vue";
import { string } from "vue-types";

declare global {
  interface Window {
    EyeDropper?: any;
  }
}

const colorReg = /^#?([a-fA-F0-9]{6}|[a-fA-F0-9]{3})$/;

export default defineComponent({
  props: {
    value: string().def("#ffffffff"),
  },
  emits: ["change"],
  setup(props, { emit }) {
    const pickr = ref<any>(null);
    const containerRef = ref();

    const state = reactive({
      showPicker: false,
      hex: "",
      opacity: 100,
      color: props.value,
    });

    function initPicker() {
      if (window.EyeDropper) state.showPicker = true;
      if (!containerRef.value) return;
      // @ts-ignore
      pickr.value = new iro.ColorPicker(containerRef.value, {
        width: 270,
        boxHeight: 150,
        color: props.value,
        sliderSize: 16,
        layout: [
          {
            component: iro.ui.Box,
          },
          {
            component: iro.ui.Slider,
            options: {
              sliderType: "hue",
              id: "hue-slider",
              handleSvg: "#handle",
              handleProps: { x: -3, y: -3 },
              handleRadius: 5,
            },
          },
          {
            component: iro.ui.Slider,
            options: {
              sliderType: "alpha",
              handleSvg: "#handle",
              handleProps: { x: -3, y: -3 },
              handleRadius: 5,
            },
          },
        ],
      });

      pickr.value.on("input:end", changePickerColor);
      pickr.value.on("input:move", changePickerColor);
    }

    function changePickerColor(color: any) {
      let res = "";
      if (color.alpha == 1) {
        res = color.hexString;
      } else {
        res = color.hex8String;
      }
      emit("change", res);
      state.color = res;
      parseColor();
    }

    function pickColor() {
      if (!window.EyeDropper) {
        return;
      }
      const eyeDropper = new window.EyeDropper();
      eyeDropper
        .open()
        .then((result: any) => {
          state.hex = result.sRGBHex;
          state.opacity = 100;
          setPickerColor();
        })
        .catch((e: any) => {
          console.log("e: ", e);
        });
    }

    const changeColor = (e: any) => {
      const value = e.target.value;
      if (value.indexOf("#") !== -1) {
        state.hex = value;
        setPickerColor();
      }
    };

    const colorBlur = () => {
      if (!colorReg.test(state.hex)) {
        parseColor();
      }
    };

    const changeOpacity = (v: any) => {
      state.opacity = v;
      setPickerColor();
    };

    function parseColor() {
      const colorObj = Color(state.color);
      const opacity = (colorObj.alpha() * 100).toFixed(0);
      const hex = colorObj.hex();
      // @ts-ignore
      state.opacity = opacity;
      state.hex = hex;
    }

    function setPickerColor() {
      if (!colorReg.test(state.hex)) return;
      const op = state.opacity / 100;
      const res = Color(state.hex).alpha(op).hexa();
      pickr.value.color.set(res);
      state.color = res;
      emit("change", state.color);
    }

    onMounted(() => {
      nextTick(() => {
        initPicker();
        parseColor();
      });
    });

    return () => {
      const { opacity, hex } = state;

      return (
        <div class={styles}>
          <div ref={containerRef}></div>
          <div class="mt-15px flex items-center justify-between">
            <Input
              bordered={false}
              value={hex}
              maxlength={7}
              class="flex-1 bg-[#3B3B3B] h-36px text-center cursor-pointer hover:bg-[#3B3B3B] focus:bg-[#3B3B3B]"
              onChange={changeColor}
              onBlur={colorBlur}
            />
            <InputNumber
              bordered={false}
              value={opacity}
              min={0}
              max={100}
              formatter={(v) => v + "%"}
              onChange={changeOpacity}
              controls={false}
              class={cx(
                inputStyle,
                "w-60px !ml-10px bg-[#3B3B3B] h-36px text-center hover:bg-[#3B3B3B] focus:bg-[#3B3B3B]"
              )}
            />
            {state.showPicker && (
              <IconPicker
                class="ml-10px text-36px bg-[#3B3B3B] rounded-2px transition cursor-pointer hover:opacity-80"
                onClick={pickColor}
              />
            )}
          </div>
          <svg style="display:none;">
            <circle
              fill="#fff"
              id="handle"
              cx="8"
              cy="8"
              r="6"
              // stroke-width="1"
              // stroke="#fff"
            ></circle>
          </svg>
        </div>
      );
    };
  },
});

const styles = css`
  .IroBox {
    border-radius: 6px !important;
  }
`;

const inputStyle = css`
  .ant-input-number-input {
    height: 36px;
    text-align: center;
    background-color: #3b3b3b;
  }
`;
