import { IconPalette } from "@/assets/icons";
import { css } from "@linaria/core";
import { Popover } from "ant-design-vue";
import { defineComponent, reactive, watch } from "vue";
import { any, bool } from "vue-types";
import ColorLib from "./ColorLib";
import { formatColor, gradientColorType } from "./ColorList";
import Panel from "./Panel";
import Color from "color";

export default defineComponent({
  props: {
    value: any<string | gradientColorType>(),
    showGradient: bool().def(false),
  },
  emits: ["change"],
  setup(props, { emit }) {
    const color = getColor();

    // @ts-ignore
    const state = reactive({
      color: color || "#ffffff",
    });
    watch(
      () => props.value,
      () => {
        const color = getColor();
        state.color = color;
      }
    );
    function getColor() {
      if (typeof props.value == "string") {
        const colorObj = Color(props.value);
        return colorObj.hexa();
      } else {
        return props.value || "#ffffff";
      }
    }

    function changeValue(data: any) {
      state.color = data;
      emit("change", data);
    }

    function storeColors() {
      const list = localStorage.getItem("latestColors");
      let colors = list ? JSON.parse(list) : ["#fff"];
      const index = colors.findIndex(
        (d: any) => formatColor(props.value || "") == formatColor(d)
      );
      if (index !== -1) return;
      colors.unshift(props.value);
      colors.splice(10);
      localStorage.setItem("latestColors", JSON.stringify(colors));
    }

    return () => {
      const { color } = state;

      return (
        <div class="space-x-10px flex items-center">
          <Popover
            // visible={true}
            color="#303030"
            overlayClassName={"editor_text_color"}
            destroyTooltipOnHide
            trigger="click"
            placement="bottomRight"
            content={
              <ColorLib
                showGradient={props.showGradient}
                value={color}
                onChange={changeValue}
              />
            }
            onVisibleChange={(v) => {
              if (!v) storeColors();
            }}
          >
            <div class={colorItem}>
              <div
                class="w-30px h-30px cursor-pointer"
                style={{ background: formatColor(color) }}
              ></div>
            </div>
          </Popover>
          <Popover
            // visible={true}
            color="#303030"
            destroyTooltipOnHide
            trigger="click"
            placement="bottomRight"
            overlayClassName={"editor_text_color"}
            content={
              <Panel
                showGradient={props.showGradient}
                onChange={changeValue}
                value={color}
              />
            }
            onVisibleChange={(v) => {
              if (!v) storeColors();
            }}
          >
            <IconPalette class="rounded bg-[#303030] text-30px !leading-0 text-light-50 cursor-pointer" />
          </Popover>
        </div>
      );
    };
  },
});

const colorItem = css`
  background-color: #fff;
  background-image: linear-gradient(
      -45deg,
      rgba(57, 76, 96, 0.15) 25%,
      transparent 25%,
      transparent 75%,
      rgba(57, 76, 96, 0.15) 75%
    ),
    linear-gradient(
      -45deg,
      rgba(57, 76, 96, 0.15) 25%,
      transparent 25%,
      transparent 75%,
      rgba(57, 76, 96, 0.15) 75%
    );

  background-position: 0 0, 6px 6px;
  background-size: 12px 12px;
  border-radius: 2px;
  overflow: hidden;
`;
