import { ModuleControl } from "queenjs";
import { reactive } from "vue";
import { EditorModule } from "../../module";
import { nanoid } from "nanoid";
export class CategoryCtrl extends ModuleControl<EditorModule> {
  state = reactive({
    result: {} as any,
    categories: [],
    syscategories: [],
  });

  init() {
    this.getUserCate();
    this.getSystemCate();
  }
  async getUserCate() {
    const res = await this.https.getCategories();
    if (res.errorNo != 200) {
      return;
    }
    this.state.result = res.result;
    this.state.categories = res.result.categories;
  }
  async getSystemCate() {
    const res = await this.https.getSysCategories();
    if (res.errorNo != 200) {
      return;
    }
    this.state.syscategories = res.result.categories;
  }
  buildCateData(
    data: any,
    changeItem: { id?: string; name: string; parent?: string }
  ) {
    let res = [] as any;
    data?.map((item: any, index: number) => {
      res[index] = {};

      if (item.id == changeItem.parent) {
        if (!item?.children) {
          item.children = [];
        }
        item.children.push({
          name: changeItem.name,
          value: changeItem.name,
          type: item.type,
          id: nanoid(),
        });
      }
      if (item.id == changeItem?.id) {
        item.value = changeItem.name;
        item.name = changeItem.name;
      }
      res[index] = { ...item };
      let arr = [] as any;
      if (Array.isArray(item.children) && item.children.length > 0) {
        arr = this.buildCateData(item.children, changeItem);
        res[index].children = arr;
      }
    });
    return res;
  }
  delCateData(data: any, id: string) {
    data?.map((item: any, index: number) => {
      if (item.id == id) {
        data.splice(index, 1);
        return;
      }
      if (Array.isArray(item.children) && item.children.length > 0) {
        this.delCateData(item.children, id);
      }
    });
    return data;
  }

  async updateCate(data: { id?: string; name: string; parent?: string }) {
    const categories: any = [...this.state.categories];
    const resData = this.buildCateData(categories, data);
    this.state.result.categories = resData;
    await this.https.updateCategories(this.state.result);
    this.state.categories = categories;
  }

  async deleteCate(id: string) {
    const categories: any = [...this.state.categories];
    const resData = this.delCateData(categories, id);
    this.state.result.categories = resData;
    await this.https.updateCategories(this.state.result);
    this.state.categories = categories;
  }
}
