import { initEditor } from "@/modules/editor";
import { Design_Page_Size } from "@/modules/editor/dicts/CompOptions";
import { cx } from "@linaria/core";
import { isPc } from "@queenjs/utils";
import { defineComponent } from "vue";

export default defineComponent(() => {
  const editor = initEditor();
  const params = new URLSearchParams(location.href.split("?")[1].split("#")[0]);
  const id = params.get("id");
  const isSys = params.get("isSys");
  const isWk = params.get("isWk");

  editor.actions.switchMode("display");

  if (id) {
    if (isWk) {
      editor.actions.initWkDesign(id);
    } else {
      editor.actions.initDesign(id, isSys);
    }

    editor.controls.wxCtrl.setup(window.location.href);

    editor.actions.on("initDesign:success", () => {
      const data = editor.store.designData;
      document.title = data.title;
      const shareData = {
        title: data.title,
        link: location.href,
        imgUrl: data.thumbnail || "",
        desc: data.desc,
      };
      editor.controls.wxCtrl.setShareData(shareData);
      editor.controls.wxCtrl.setShare(shareData);
      
      editor.controls.screenCtrl.applyScreen();

    });
  }
  // fetch("https://restapi.amap.com/v3/ip?key=6f53b2e09f72ad63423b2da6e08b25d7").then(response=>{
  //    return response.json();
  // }).then(data=>{
  //   console.log(data);
  // })

  return () => (
    <div
      class={cx(
        isPc() && "h-100vh",
        "flex items-center justify-center bg-gray-100"
      )}
    >
      <div
        class={isPc() ? `scrollbar !overflow-x-hidden` : `overflow-x-hidden`}
        style={{
          height: editor.helper.designToNaturalSize(Design_Page_Size[1], {
            adaptiveH: true,
          }),
        }}
      >
        <editor.components.Preview />
      </div>
    </div>
  );
});
